package com.bsoft.api.service.Impl;

import com.bsoft.api.mapper.SerDiseaseMapper;
import com.bsoft.api.model.SerDisease;
import com.bsoft.api.model.respmodel.DiseaseLevel;
import com.bsoft.api.service.SerDiseaseService;
import com.bsoft.common.utils.StringUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class SerDiseaseServiceImpl implements SerDiseaseService {
    @Resource
    private SerDiseaseMapper serDiseaseMapper;

    @Override
    public List<DiseaseLevel> selectListByIdorLevel(String date,String diseaseCode,Integer level,String docCode,String deptCode,Long userId) {
        List<DiseaseLevel> list = new ArrayList<>();
        docCode = StringUtil.isNullOrEmpty(docCode) ? null : docCode;
        deptCode = StringUtil.isNullOrEmpty(deptCode) ? null : deptCode;
        if(diseaseCode != null && diseaseCode != ""){
            list = selectByCode(date,diseaseCode,docCode,deptCode);
        }else{
            list = selectByLevel(date,level,docCode,deptCode,userId);
        }
        return list;
    }

    @Override
    public List<SerDisease> selectByMdcName(String date,String mdcName,String docCode,String deptCode) {
        mdcName = StringUtil.isNullOrEmpty(mdcName) ? null : mdcName;
        docCode = StringUtil.isNullOrEmpty(docCode) ? null : docCode;
        deptCode = StringUtil.isNullOrEmpty(deptCode) ? null : deptCode;
        return serDiseaseMapper.selectByMdcName(date,mdcName,docCode,deptCode);
    }

    /**
     * 根据Code查询父子级别列表
     *
     * @param date 时间
     * @param diseaseCode ID
     * @return
     */
    private List<DiseaseLevel> selectByCode(String date,String diseaseCode,String docCode,String deptCode) {


        SerDisease disease = serDiseaseMapper.selectByCode(diseaseCode,date,docCode,deptCode);
        if(disease == null){
            return null;
        }

        boolean isParent = true;
        DiseaseLevel diseaseLevelResult = null;
        Integer diseaseId = disease.getId().intValue();
        //查询父级菜单对象(直到查询到顶级)
        while(isParent){
            DiseaseLevel diseaseLevel = serDiseaseMapper.selectParentById(date,diseaseId);
            if(diseaseLevel != null){
                List<DiseaseLevel> sonList = new ArrayList<>();
                if(diseaseLevelResult != null){
                    sonList.add(diseaseLevelResult);
                }
                diseaseLevel.setDiseaseLevelList(sonList);
                diseaseId = diseaseLevel.getId().intValue();
                diseaseLevelResult = diseaseLevel;
            }else{
                isParent = false;
            }
        }

        List<DiseaseLevel> parentList = new ArrayList<>();
        //查询同级菜单列表
        List<DiseaseLevel> serDiseaseList = serDiseaseMapper.selectByParentId(date,disease.getParentId().intValue(),docCode,deptCode);
        List<DiseaseLevel> diseaseLevelList = new ArrayList<>();
        for(DiseaseLevel serDiseases : serDiseaseList){
            DiseaseLevel diseaseLevel1 = new DiseaseLevel();
            //查询子级菜单列表
            if(serDiseases.getMdcCode().equals(diseaseCode)){
                List<DiseaseLevel> diseaseLevelList1 = serDiseaseMapper.selectSonById(date,disease.getId().intValue(),docCode,deptCode);
                diseaseLevel1.setDiseaseLevelList(diseaseLevelList1);
            }
            diseaseLevel1.setDate(serDiseases.getDate());
            diseaseLevel1.setId(serDiseases.getId());
            diseaseLevel1.setLevel(serDiseases.getLevel());
            diseaseLevel1.setMdcCode(serDiseases.getMdcCode());
            diseaseLevel1.setMdcName(serDiseases.getMdcName());
            diseaseLevel1.setMdcNum(serDiseases.getMdcNum());
            diseaseLevel1.setOrgId(serDiseases.getOrgId());
            diseaseLevel1.setOrgName(serDiseases.getOrgName());
            diseaseLevel1.setParentId(serDiseases.getParentId());
            diseaseLevelList.add(diseaseLevel1);
        }
        if(diseaseLevelResult != null){
            diseaseLevelResult = getSonDiseaseList(diseaseLevelResult,null,diseaseLevelList);
//            diseaseLevelResult.setDiseaseLevelList(diseaseLevelList);
            parentList.add(diseaseLevelResult);
            return parentList;
        }else{
            return diseaseLevelList;
        }
    }


    private DiseaseLevel getSonDiseaseList(DiseaseLevel diseaseLevel,List<DiseaseLevel> diseaseLevelList,
                                           List<DiseaseLevel> diseaseLevelListSon) {
        if(diseaseLevel != null && diseaseLevel.getDiseaseLevelList() != null && diseaseLevel.getDiseaseLevelList().size() > 0){
            getSonDiseaseList(null,diseaseLevel.getDiseaseLevelList(),diseaseLevelListSon);
        }else if(diseaseLevelList != null && diseaseLevelList.size() > 0){
            for(DiseaseLevel info : diseaseLevelList){
                getSonDiseaseList(info,null,diseaseLevelListSon);
            }
        }else{
            if(diseaseLevel != null && diseaseLevelListSon != null && diseaseLevelListSon.size() > 0){
                diseaseLevel.setDiseaseLevelList(diseaseLevelListSon);
            }
        }
        return diseaseLevel;
    }

    /**
     * 根据等级查询所有该等级数据
     *
     * @param date
     * @param level
     * @return
     */
    private List<DiseaseLevel> selectByLevel(String date,Integer level,String docCode,String deptCode,Long userId) {
        if(level == null){
            level = 0;
        }
        return serDiseaseMapper.selectByLevel(date,level,docCode,deptCode,null,userId);
    }
}
